-------------------------------------------------------------------------------
-- Title      : controlUnit
-- Project    :
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    :
-- Created    : 2018-04-11
-- Last update: 2019-02-13
-- Platform   :
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Control unit of a sequential FIR filter.
-------------------------------------------------------------------------------
-- Copyright (c) 2018
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2018-04-11  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity controlUnit is

  port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSampleValid    : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_processingDone_y    : in  std_logic;
    I_processingDone_z_a  : in  std_logic;
    I_processingDone_z_b  : in  std_logic;
    I_processingDone_r    : in  std_logic;
    O_loadShift_X         : out std_logic;  -- 
    O_loadShift_Y         : out std_logic;  -- 
    O_loadShift_Z         : out std_logic;  -- 
    O_initAddress_x       : out std_logic;  -- Control signal to initialize register read address
    O_initAddress_y       : out std_logic;  
    O_initAddress_z       : out std_logic;  
    O_incrAddress_x       : out std_logic;  -- Control signal to increment register read address
    O_incrAddress_y       : out std_logic;  -- Control signal to increment register read address
    O_incrAddress_z       : out std_logic;  -- Control signal to increment register read address
    O_filtre              : out std_logic_vector(1 downto 0);
    O_filtreCoef          : out std_logic_vector(1 downto 0);
    O_initSum             : out std_logic;  -- Control signal to initialize the MAC register
    O_loadSum             : out std_logic;  -- Control signal to load the MAC register;
    O_loadY               : out std_logic;  -- Control signal to load Y register
    O_FilteredSampleValid : out std_logic  -- Data valid signal for filtered sample
    );

end entity controlUnit;
architecture archi_operativeUnit of controlUnit is


  type T_state is (WAIT_SAMPLE, STORE_X, PROCESSING_LOOP_Y, STORE_Y, PROCESSING_LOOP_Z_A, INITADRESS, PROCESSING_LOOP_Z_B, STORE_Z, PROCESSING_LOOP_R, OUTPUT, WAIT_END_SAMPLE);  -- state list
  signal SR_presentState : T_state;
  signal SR_futurState   : T_state;

begin

  process (I_clock, I_reset) is
  begin
    if I_reset = '1' then               -- asynchronous reset (active high)
      SR_presentState <= WAIT_SAMPLE;
    elsif rising_edge(I_clock) then     -- rising clock edge
      SR_presentState <= SR_futurState;
    end if;
  end process;

  process (I_clock, SR_presentState) is
  begin
    case SR_presentState is
      when WAIT_SAMPLE =>
        if I_inputSampleValid = '0' then
            SR_futurState <= WAIT_SAMPLE;
        else
            SR_futurState <= STORE_X;
        end if;
      
      when STORE_X =>
        SR_futurState <= PROCESSING_LOOP_Y;
        
      when PROCESSING_LOOP_Y =>
        if I_processingDone_Y = '1' then
            SR_futurState <= STORE_Y;
        else
            SR_futurSTATE <= PROCESSING_LOOP_Y;
        end if;
        
      when STORE_Y =>
        SR_futurState <= PROCESSING_LOOP_Z_A;
        
      when PROCESSING_LOOP_Z_A =>
        if I_processingDone_z_a = '1' then
            SR_futurState <= INITADRESS;
        else
            SR_futurState <= PROCESSING_LOOP_Z_A;
        end if;
        
      when INITADRESS =>
        SR_futurState <= PROCESSING_LOOP_Z_B;
        
      when PROCESSING_LOOP_Z_B =>
        if I_processingDone_z_b = '1' then
            SR_futurState <= STORE_Z;
        else
            SR_futurState <= PROCESSING_LOOP_Z_B;
        end if;
        
      when STORE_Z =>
        SR_futurState <= PROCESSING_LOOP_R;
        
      when PROCESSING_LOOP_R =>
        if I_processingDone_R = '1' then
            SR_futurState <= OUTPUT;
        else
            SR_futurState <= PROCESSING_LOOP_R;
        end if;
        
      when OUTPUT =>
        SR_futurState <= WAIT_END_SAMPLE;
        
      when WAIT_END_SAMPLE =>
        if I_inputSampleValid = '1' then
            SR_futurState <= WAIT_END_SAMPLE;
        else
            SR_futurState <= WAIT_SAMPLE;
        end if;

      when others => null;
    end case;
  end process;
  
  handle_filters: process (I_clock, I_reset, SR_presentState) is
  begin
    if I_reset = '1' then
        O_Filtre     <= "00";
        O_FiltreCoef <= "00";
    else
        if SR_presentState = STORE_X or SR_presentState = PROCESSING_LOOP_Y then
            O_Filtre     <= "00";
            O_FiltreCoef <= "00";
        end if;
        
        if SR_presentState = PROCESSING_LOOP_Z_B then
            O_Filtre     <= "01";
            O_FiltreCoef <= "01";
        end if;
        
        if SR_presentState = PROCESSING_LOOP_Z_A then
            O_filtre     <= "10";      
        end if;
        
        if SR_presentState = PROCESSING_LOOP_R then
            O_Filtre     <= "10";
            O_FiltreCoef <= "11";
        end if;
    end if;
  end process handle_filters;

  O_loadShift_X         <= '1' when SR_presentState = STORE_X else '0';
  O_loadShift_Y         <= '1' when SR_presentState = STORE_Y else '0';
  O_loadShift_Z         <= '1' when SR_presentState = STORE_Z else '0';
  O_initAddress_x       <= '1' when SR_presentState = STORE_X else '0';
  O_initAddress_y       <= '1' when SR_presentState = STORE_Y or SR_presentState = INITADRESS else '0';
  O_initAddress_z       <= '1' when SR_presentState = STORE_Z else '0';
  O_incrAddress_x       <= '1' when SR_presentState = PROCESSING_LOOP_Y else '0';
  O_incrAddress_y       <= '1' when SR_presentState = PROCESSING_LOOP_Z_A or SR_presentState = PROCESSING_LOOP_Z_B else '0';
  O_incrAddress_z       <= '1' when SR_presentState = PROCESSING_LOOP_R else '0';
  O_initSum             <= '1' when SR_presentState = STORE_X
                                 or SR_presentState = STORE_Y
                                 or SR_presentState = STORE_Z else '0';
  O_loadSum             <= '1' when SR_presentState = PROCESSING_LOOP_Y
                                 or SR_presentState = PROCESSING_LOOP_Z_A
                                 or SR_presentState = PROCESSING_LOOP_Z_B
                                 or SR_presentState = PROCESSING_LOOP_R else '0';
                                                                  
  O_loadY               <= '1' when SR_presentState = OUTPUT else '0';
--  O_FilteredSampleValid <= '1' when _BLANK_ ;

end architecture archi_operativeUnit;
