-------------------------------------------------------------------------------
-- Title      : firUnit
-- Project    : 
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    : 
-- Created    : 2018-04-11
-- Last update: 2018-04-11
-- Platform   : 
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 8 bit FIR
-------------------------------------------------------------------------------
-- Copyright (c) 2018 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2018-04-11  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity firUnit is

  port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSample         : in  std_logic_vector(10 downto 0);  -- 8 bit input sample
    I_inputSampleValid    : in  std_logic;
    O_filteredSample      : out std_logic_vector(10 downto 0);  -- filtered sample
    O_filteredSampleValid : out std_logic
    );

end entity firUnit;

architecture archi_firUnit of firUnit is

  component controlUnit is
    port (
      I_clock               : in  std_logic;
      I_reset               : in  std_logic;
      I_inputSampleValid    : in  std_logic;
      I_processingDone_Y    : in  std_logic;
      I_processingDone_Za   : in  std_logic;
      I_processingDone_Zb   : in  std_logic;
      I_processingDone_R    : in  std_logic;
      O_loadShift_X         : out std_logic;  -- 
      O_loadShift_Y         : out std_logic;  -- 
      O_loadShift_Z         : out std_logic;  -- 
      O_initAddress_x       : in  std_logic;   -- Control signal to initialize register read address
      O_initAddress_y       : in  std_logic;   -- Control signal to initialize register read address
      O_initAddress_z       : in  std_logic;   -- Control signal to initialize register read address
      O_incrAddress_Y       : out std_logic;  -- Control signal to increment register read address
      O_incrAddress_Za      : out std_logic;  -- Control signal to increment register read address
      O_incrAddress_Zb      : out std_logic;  -- Control signal to increment register read address
      O_filtre              : out std_logic_vector(1 downto 0);
      O_filtreCoef          : out std_logic_vector(1 downto 0);
      O_initSum             : out std_logic;
      O_loadSum             : out std_logic;
      O_loadY               : out std_logic;
      O_FilteredSampleValid : out std_logic);
  end component controlUnit;

  component operativeUnit is
    port (
      I_clock               : in  std_logic;
      I_reset               : in  std_logic;
      I_inputSample         : in  std_logic_vector(10 downto 0);
      I_loadShift_x         : in  std_logic;   -- Control signal to load the input sample in the sample shift register and shift the register
      I_loadShift_y         : in  std_logic;   -- Control signal to load the input sample in the sample shift register and shift the register
      I_loadShift_z         : in  std_logic;   -- Control signal to load the input sample in the sample shift register and shift the register
      I_initAddress_x       : in  std_logic;   -- Control signal to initialize register read address
      I_initAddress_y       : in  std_logic;   -- Control signal to initialize register read address
      I_initAddress_z       : in  std_logic;   -- Control signal to initialize register read address
      I_incrAddress_x       : in  std_logic;   -- Control signal to increment register read address
      I_incrAddress_y       : in  std_logic;   -- Control signal to increment register read address
      I_incrAddress_z       : in  std_logic;   -- Control signal to increment register read address
      I_initSum             : in  std_logic;
      I_loadSum             : in  std_logic;
      I_loadY               : in  std_logic;
      I_filtre              : in  std_logic_vector(1 downto 0);
      I_filtreCoef          : in  std_logic_vector(1 downto 0);
      O_processingDone_y    : out std_logic;   -- Indicate that processing is done
      O_processingDone_z_a  : out std_logic;   -- Indicate that processing is done
      O_processingDone_z_b  : out std_logic;   -- Indicate that processing is done
      O_processingDone_r    : out std_logic;   -- Indicate that processing is done
      O_Y                   : out std_logic_vector(10 downto 0));
  end component operativeUnit;

  signal SC_processingDone_y   : std_logic;
  signal SC_processingDone_z_a : std_logic;
  signal SC_processingDone_z_b : std_logic;
  signal SC_processingDone_r   : std_logic;
  signal SC_loadShift_x        : std_logic;
  signal SC_loadShift_y        : std_logic;
  signal SC_loadShift_Z        : std_logic;
  signal SC_initAddress_x      : std_logic;
  signal SC_initAddress_y      : std_logic;
  signal SC_initAddress_z      : std_logic;
  signal SC_incrAddress_x      : std_logic;
  signal SC_incrAddress_y      : std_logic;
  signal SC_incrAddress_z      : std_logic;
  signal SC_initSum            : std_logic;
  signal SC_Filtre             : std_logic_vector(1 downto 0);
  signal SC_FiltreCoef         : std_logic_vector(1 downto 0);
  signal SC_loadSum            : std_logic;
  signal SC_loadY              : std_logic;

begin

  controlUnit_1 : entity work.controlUnit
    port map (
      I_clock               => I_clock,
      I_reset               => I_reset,
      I_inputSampleValid    => I_inputSampleValid,
      I_processingDone_y    => SC_processingDone_y,
      I_processingDone_z_a  => SC_processingDone_z_a,
      I_processingDone_z_b  => SC_processingDone_z_b,
      I_processingDone_r    => SC_processingDone_r,
      O_loadShift_x         => SC_loadShift_x,
      O_initAddress_x       => SC_initAddress_x,
      O_initAddress_y       => SC_initAddress_y,
      O_initAddress_z       => SC_initAddress_z,
      O_incrAddress_x       => SC_incrAddress_x,
      O_incrAddress_y       => SC_incrAddress_y,
      O_incrAddress_z       => SC_incrAddress_z,
      O_filtre              => SC_Filtre,
      O_filtreCoef          => SC_FiltreCoef,
      O_initSum             => SC_initSum,
      O_loadSum             => SC_loadSum,
      O_loadY               => SC_loadY,
      O_FilteredSampleValid => O_FilteredSampleValid);

  operativeUnit_1 : entity work.operativeUnit
    port map (
      I_clock              => I_clock,
      I_reset              => I_reset,
      I_inputSample        => I_inputSample,
      I_loadShift_x        => SC_loadShift_x,
      I_loadShift_y        => SC_loadShift_y,
      I_loadShift_z        => SC_loadShift_z,
      I_initAddress_x      => SC_initAddress_x,
      I_initAddress_y      => SC_initAddress_y,
      I_initAddress_z      => SC_initAddress_z,
      I_incrAddress_x      => SC_incrAddress_x,
      I_incrAddress_y      => SC_incrAddress_y,
      I_incrAddress_z      => SC_incrAddress_z,
      I_initSum            => SC_initSum,
      I_loadSum            => SC_loadSum,
      I_loadY              => SC_loadY,
      I_Filtre             => SC_Filtre,
      I_FiltreCoef         => SC_FiltreCoef,
      O_processingDone_y   => SC_processingDone_y,
      O_processingDone_z_a => SC_processingDone_z_a,
      O_processingDone_z_b => SC_processingDone_z_b,
      O_processingDone_r   => SC_processingDone_r,
      O_Y                  => O_filteredSample);

end architecture archi_firUnit;
