-------------------------------------------------------------------------------
-- Title      : operativeUnit
-- Project    :
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    :
-- Created    : 2018-04-11
-- Last update: 2019-02-13
-- Platform   :
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: Operative unit of a sequential FIR filter. Including shift
-- register for samples, registers for coefficients, a MAC and a register to
-- store the result
-------------------------------------------------------------------------------
-- Copyright (c) 2018
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2019-02-13  1.1      marzel  Update to provide a 16-tap filter and improve
--                              the user experience ;)
-- 2018-04-11  1.0      jnbazin Created
-- 2018-04-18  1.0      marzel  Modification of SR_Y assignment to a round
--                              instead of a trunc
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity operativeUnit is

  port (
    I_clock               : in  std_logic;   -- global clock
    I_reset               : in  std_logic;   -- asynchronous global reset
    I_inputSample         : in  std_logic_vector(10 downto 0);  -- 8 bit input sample
    I_loadShift_x         : in  std_logic;   -- Control signal to load the input sample in the sample shift register and shift the register
    I_loadShift_y         : in  std_logic;   -- Control signal to load the input sample in the sample shift register and shift the register
    I_loadShift_z         : in  std_logic;   -- Control signal to load the input sample in the sample shift register and shift the register
    I_initAddress_x       : in  std_logic;   -- Control signal to initialize register read address
    I_initAddress_y       : in  std_logic;   -- Control signal to initialize register read address
    I_initAddress_z       : in  std_logic;   -- Control signal to initialize register read address
    I_incrAddress_x       : in  std_logic;   -- Control signal to increment register read address
    I_incrAddress_y       : in  std_logic;   -- Control signal to increment register read address
    I_incrAddress_z       : in  std_logic;   -- Control signal to increment register read address
    I_initSum             : in  std_logic;   -- Control signal to initialize the MAC register
    I_loadSum             : in  std_logic;   -- Control signal to load the MAC register;
    I_loadY               : in  std_logic;   -- Control signal to load Y register
    I_filtre              : in  std_logic_vector(1 downto 0);
    I_filtreCoef          : in  std_logic_vector(1 downto 0);
    O_processingDone_y    : out std_logic := '0';   -- Indicate that processing is done
    O_processingDone_z_a  : out std_logic := '0';   -- Indicate that processing is done
    O_processingDone_z_b  : out std_logic := '0';   -- Indicate that processing is done
    O_processingDone_r    : out std_logic := '0';   -- Indicate that processing is done
    O_Y                   : out std_logic_vector(10 downto 0)   -- filtered sample
    );

end entity operativeUnit;

architecture arch_operativeUnit of operativeUnit is
  type registerFile_x is array(0 to 94) of signed(10 downto 0);
  type registerFile_y is array(0 to 2)  of signed(10 downto 0);
  type registerFile_z is array(0 to 10) of signed(10 downto 0);
  
  -- COEFFICIENTS
  signal SR_coefRegister_h : registerFile_x;
  signal SR_coefRegister_b : registerFile_y;
  signal SR_coefRegister_a : registerFile_y;
  signal SR_coefRegister_g : registerFile_z;

  signal SR_shiftRegister_x : registerFile_x;  -- shift register file used to store and shift input samples
  signal SR_shiftRegister_y : registerFile_y;
  signal SR_shiftRegister_z : registerFile_z;
  
  signal SC_multOperand1  : signed(10 downto 0);
  signal SC_multOperand2  : signed(10 downto 0);
  signal SC_MultResult    : signed(21 downto 0);  -- Result of the multiplication Xi*Hi
  signal SC_addResult     : signed(28 downto 0);  -- result of the accumulation addition
  signal SR_sum           : signed(28 downto 0);  -- Accumulation register
  signal SR_Y             : signed(10 downto 0);  -- filtered sample storage register
  
  -- ADDRESS GENERATORS
  signal SR_readAddress_1 : integer range 0 to 94;  -- register files read address
  signal SR_readAddress_2 : integer range 0 to 2;
  signal SR_readAddress_3 : integer range 0 to 2;
  signal SR_readAddress_4 : integer range 0 to 10;

begin

-- BaseLine filter provided with octave (or Matlab ;)) command
-- fir1(128, 1/100, 'high')*2^10
  SR_coefRegister_h <= (to_signed(-1, 11),  -- ROM register used file to store FIR coefficients
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-2, 11),
                        to_signed(-2, 11),
                        to_signed(-2, 11),
                        to_signed(-3, 11),
                        to_signed(-3, 11),
                        to_signed(-3, 11),
                        to_signed(-4, 11),
                        to_signed(-4, 11),
                        to_signed(-5, 11),
                        to_signed(-5, 11),
                        to_signed(-6, 11),
                        to_signed(-6, 11),
                        to_signed(-7, 11),
                        to_signed(-7, 11),
                        to_signed(-8, 11),
                        to_signed(-8, 11),
                        to_signed(-9, 11),
                        to_signed(-10, 11),
                        to_signed(-10, 11),
                        to_signed(-11, 11),
                        to_signed(-11, 11),
                        to_signed(-12, 11),
                        to_signed(-13, 11),
                        to_signed(-13, 11),
                        to_signed(-14, 11),
                        to_signed(-14, 11),
                        to_signed(-15, 11),
                        to_signed(-15, 11),
                        to_signed(-16, 11),
                        to_signed(-16, 11),
                        to_signed(-17, 11),
                        to_signed(-17, 11),
                        to_signed(-18, 11),
                        to_signed(-18, 11),
                        to_signed(-18, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(1004, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-19, 11),
                        to_signed(-18, 11),
                        to_signed(-18, 11),
                        to_signed(-18, 11),
                        to_signed(-17, 11),
                        to_signed(-17, 11),
                        to_signed(-16, 11),
                        to_signed(-16, 11),
                        to_signed(-15, 11),
                        to_signed(-15, 11),
                        to_signed(-14, 11),
                        to_signed(-14, 11),
                        to_signed(-13, 11),
                        to_signed(-13, 11),
                        to_signed(-12, 11),
                        to_signed(-11, 11),
                        to_signed(-11, 11),
                        to_signed(-10, 11),
                        to_signed(-10, 11),
                        to_signed(-9, 11),
                        to_signed(-8, 11),
                        to_signed(-8, 11),
                        to_signed(-7, 11),
                        to_signed(-7, 11),
                        to_signed(-6, 11),
                        to_signed(-6, 11),
                        to_signed(-5, 11),
                        to_signed(-5, 11),
                        to_signed(-4, 11),
                        to_signed(-4, 11),
                        to_signed(-3, 11),
                        to_signed(-3, 11),
                        to_signed(-3, 11),
                        to_signed(-2, 11),
                        to_signed(-2, 11),
                        to_signed(-2, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11),
                        to_signed(-1, 11)
                        );
                       
-- Pei Tseng Notch filter provided by octave
-- [b, a] = pei_tseng_notch(1/5, 1/25)*2^9
SR_coefRegister_b <= (
                        to_signed(480, 11),
                        to_signed(-777, 11),
                        to_signed(480, 11)
                        );

SR_coefRegister_a <= (
                        to_signed(512, 11),
                        to_signed(-777, 11),
                        to_signed(449, 11)
                        );

-- Parks McClellan filter provided by octave
-- fLP = remez(10, [0 50 60 250]/250, [1 1 0 0])*2^10
SR_coefRegister_g <= (
                        to_signed(-119, 11),
                        to_signed(122, 11),
                        to_signed(149, 11),
                        to_signed(191, 11),
                        to_signed(226, 11),
                        to_signed(239, 11),
                        to_signed(226, 11),
                        to_signed(191, 11),
                        to_signed(149, 11),
                        to_signed(122, 11),
                        to_signed(-119, 11)
                        );

  shift : process (I_clock, I_reset) is
  begin  -- process shift
    if I_reset = '1' then               -- asynchronous reset (active high)
        SR_shiftRegister_x <= (others => (others => '0'));
        SR_shiftRegister_y <= (others => (others => '0'));
        SR_shiftRegister_z <= (others => (others => '0'));
    elsif rising_edge(I_clock) then
        if I_loadShift_x = '1' then
            for i in 94 downto 1 loop
                SR_shiftRegister_x(i) <= SR_shiftRegister_x(i-1);
            end loop;
            SR_shiftRegister_x(0) <= signed(I_inputSample);
        end if;
        
        if I_loadShift_y = '1' then
            for i in 2 downto 1 loop
                SR_shiftRegister_y(i) <= SR_shiftRegister_y(i-1);
            end loop;
            SR_shiftRegister_y(0) <= SR_sum(21 downto 11);
        end if;
        
        if I_loadShift_z = '1' then
            for i in 10 downto 1 loop
                SR_shiftRegister_z(i) <= SR_shiftRegister_z(i-1);
            end loop;
            SR_shiftRegister_z(0) <= SR_sum(21 downto 11);
        end if;
    end if;
  end process shift;

  incr_address : process (I_clock, I_reset) is
  begin
    if I_reset = '1' then               -- asynchronous reset (active high)
      -- REINITIALISATION DES ADRESSES
      SR_readAddress_1 <= 0;
      SR_readAddress_2 <= 0;
      SR_readAddress_3 <= 0;
      SR_readAddress_4 <= 0;
    elsif rising_edge(I_clock) then
        -- PREMIER FILTRE Y
        if I_initAddress_x = '1' then
            SR_readAddress_1 <= 0;
        elsif I_incrAddress_x = '1' then
            if SR_readAddress_1 < 94 then
                SR_readAddress_1 <= SR_readAddress_1 + 1;
            else
                SR_readAddress_1 <= 0;
            end if;
        end if;
        
        -- SECOND FILTRE (AVEC DEUX COEFFICIENTS)
        if I_initAddress_y = '1' then
            SR_readAddress_2 <= 0;
            SR_readAddress_3 <= 0;
        elsif I_incrAddress_y = '1' then
            if SR_readAddress_2 < 2 then
                SR_readAddress_2 <= SR_readAddress_2 + 1;
            else
                SR_readAddress_2 <= 0;
            end if;
            
            if SR_readAddress_3 < 2 then
                SR_readAddress_3 <= SR_readAddress_3 + 1;
            else
                SR_readAddress_3 <= 0;
            end if;
        end if;
        
        -- TROISIEME FILTRE
        if I_initAddress_z = '1' then
            SR_readAddress_4 <= 0;
        elsif I_incrAddress_z = '1' then
            if SR_readAddress_4 < 10 then
                SR_readAddress_4 <= SR_readAddress_4 + 1;
            else
                SR_readAddress_4 <= 0;
            end if;
        end if;
    end if;
  end process incr_address;

  O_processingDone_y   <= '1' when SR_readAddress_1 = 94 else '0'; -- CALCUL DE Y TERMINÉ
  O_processingDone_z_a <= '1' when SR_readAddress_2 = 2  else '0';  -- CALCUL DE Z_A TERMINÉ
  O_processingDone_z_b <= '1' when SR_readAddress_3 = 2  else '0'; -- CALCUL DE Z_B TERMINÉ
  O_processingDone_r   <= '1' when SR_readAddress_4 = 10 else '0'; -- CALCUL DE R TERMINÉ

-- A VERIFIER
mult_choose: process (I_clock, I_reset) is
begin
    if I_reset = '1' then
        SC_multOperand1 <= (others => '0');
        SC_multOperand1(0) <= '1';
        
        SC_multOperand2 <= (others => '0');
        SC_multOperand2(0) <= '1';
    else
        case I_filtre is
            when "00" =>
                SC_multOperand1 <= SR_shiftRegister_x(SR_readAddress_1);
            when "01" =>
                SC_multOperand1 <= SR_shiftRegister_y(SR_readAddress_2);
            when "10" =>
                SC_multOperand1 <= SR_shiftRegister_z(SR_readAddress_4);
            when others =>
                SC_multOperand1 <= (others => '0');
                SC_multOperand1(0) <= '1';
        end case; 
        
        case I_filtreCoef is
            when "00" =>
                SC_multOperand2 <= SR_coefRegister_h(SR_readAddress_1);
            when "01" =>
                SC_multOperand2 <= SR_coefRegister_b(SR_readAddress_2);
            when "10" =>
                SC_multOperand2 <= SR_coefRegister_a(SR_readAddress_3);
            when "11" =>
                SC_multOperand2 <= SR_coefRegister_g(SR_readAddress_4);
            when others =>
                SC_multOperand2 <= (others => '0');
                SC_multOperand2(0) <= '1';
        end case; 
    end if;
end process mult_choose;

  SC_MultResult   <= SC_multOperand1 * SC_multOperand2 ;  -- 22 bits
  SC_addResult    <= resize(SC_MultResult, SC_addResult'length) + SR_sum;

  sum_acc : process (I_clock, I_reset) is
  begin
    if I_reset = '1' then               -- asynchronous reset (active high)
      SR_sum <= (others => '0');
    elsif rising_edge(I_clock) then
        if I_initSum = '1' then
            SR_sum <= (others => '0');
        elsif I_loadSum = '1' then
            SR_sum <= SC_addResult;
        end if;
    end if;
  end process sum_acc;

  store_result : process (I_clock, I_reset) is
  begin
      if I_reset = '1' then
        SR_Y <= (others => '0');
      elsif rising_edge(I_clock) then
        if I_loadY = '1' then
            SR_Y <= SC_addResult(20 downto 10);
        end if;
      end if;

  end process store_result;

  O_Y <= std_logic_vector(SR_Y);

end architecture arch_operativeUnit;
